import type { Dictionary, Nullable } from './helper-types';
import type { BrowserBuildFormat, RollupSourcemap } from './types';
export declare type BuildTypeForAPI = 'main' | 'module' | 'browser';
export declare type BuildType = BuildTypeForAPI | 'bin';
export declare type GlobalsOptions = Nullable<Dictionary<string> | string[]>;
export interface WithSourcemapOption {
    sourcemap?: Nullable<RollupSourcemap>;
}
export interface WithModuleOptions {
    esModule?: Nullable<boolean>;
    interop?: Nullable<boolean>;
}
export interface WithMinOption {
    min?: Nullable<boolean>;
}
export interface InputOptions {
    api?: Nullable<string>;
    bin?: Nullable<string>;
}
export declare type ModuleString = Exclude<BuildType, 'module'>;
export declare type ModuleOption = Nullable<ModuleString | ModuleString[] | boolean>;
export declare type MinOption = Nullable<BuildType | BuildType[] | boolean>;
export interface TypesOptions {
    equals?: Nullable<boolean>;
}
export interface BrowserOptions {
    format?: Nullable<BrowserBuildFormat>;
    name?: Nullable<string>;
    id?: Nullable<string>;
    extend?: Nullable<boolean>;
    globals?: GlobalsOptions;
}
export interface CommonJSBuildOptions {
    sourcemap?: Nullable<RollupSourcemap>;
    esModule?: Nullable<boolean>;
    interop?: Nullable<boolean>;
    min?: Nullable<boolean>;
}
export interface ESModuleBuildOptions {
    sourcemap?: Nullable<RollupSourcemap>;
    min?: Nullable<boolean>;
}
export interface BrowserBuildOptions {
    sourcemap?: Nullable<RollupSourcemap>;
    esModule?: Nullable<boolean>;
    interop?: Nullable<boolean>;
    min?: Nullable<boolean>;
    format?: Nullable<BrowserBuildFormat>;
    name?: Nullable<string>;
    id?: Nullable<string>;
    extend?: Nullable<boolean>;
    globals?: GlobalsOptions;
}
export declare type SelectiveType<K extends string> = BuildTypeForAPI | K | 'api';
export declare type ObjectSelectiveOptionsKey<K extends string> = SelectiveType<K> | 'default';
export declare type ObjectSelectiveOptions<K extends string, T> = Partial<Record<ObjectSelectiveOptionsKey<K>, Nullable<T>>>;
export declare type StringBasedSelectiveOption<K extends string> = SelectiveType<K> | Array<SelectiveType<K>>;
export declare type ObjectBasedSelectiveOption<K extends string, T> = Nullable<T> | ObjectSelectiveOptions<K, T>;
export declare type SelectiveOption<K extends string, T> = StringBasedSelectiveOption<K> | ObjectBasedSelectiveOption<K, T>;
export declare type SelectiveSkipBuildType = BuildType | 'types';
export declare type SelectiveBooleanOption = SelectiveOption<BuildType, boolean>;
export declare type SelectiveStringOption = ObjectBasedSelectiveOption<BuildType, string>;
export declare type SelectiveSourcemap = SelectiveOption<BuildType, RollupSourcemap>;
export declare type SelectiveSkipOption = SelectiveOption<SelectiveSkipBuildType, boolean>;
export interface DeprecatedBundlibOptions {
    main?: Nullable<CommonJSBuildOptions | false>;
    module?: Nullable<ESModuleBuildOptions | false>;
    browser?: Nullable<BrowserBuildOptions | false>;
    bin?: Nullable<CommonJSBuildOptions | string | false>;
    types?: Nullable<TypesOptions>;
}
export interface BundlibOptions extends DeprecatedBundlibOptions {
    input?: Nullable<SelectiveStringOption>;
    sourcemap?: SelectiveSourcemap;
    esModule?: SelectiveBooleanOption;
    interop?: SelectiveBooleanOption;
    min?: SelectiveBooleanOption;
    cache?: Nullable<string>;
    project?: SelectiveStringOption;
    skip?: SelectiveSkipOption;
    format?: Nullable<BrowserBuildFormat>;
    name?: Nullable<string>;
    id?: Nullable<string>;
    extend?: Nullable<boolean>;
    globals?: GlobalsOptions;
    equals?: Nullable<boolean>;
}
